<?php

/**
 * Admin settings for wordpress plugin
 * @author 
 */
class KB_Admin extends KB_Wordpress
{
	const GENERAL_OPTIONS_PAGE = 'options-general.php';

	public function __construct()
	{
		parent::__construct();
		
		// Add page to the Admin settings menu
		add_action('admin_menu', array($this, 'addAdminMenu'), 1);
			
		// Add Thickbox Media Uploader actions to WordPress
		add_action('admin_print_scripts', array($this, 'my_admin_scripts'));
		add_action('admin_print_styles', array($this, 'my_admin_styles'));
		
		add_action('save_post', array('KB_Posts', 'savePostCallback'), 10, 2);
		add_action('deleted_post', array('KB_Posts', 'deletePostCallback'), 11);

		// Set Ajax calls
		if (KB_Main::isAjaxCall())
		{
			$this->_setDefaultAjax();
		}

		// Enqueue all the scripts here since we need to do this after the plug-in has finished initialization
		add_action('admin_init', array($this, 'admin_init'));
		
		add_action('load-post.php', array($this, 'mceFullScreenButtonsStyle'));
		add_action('load-post-new.php', array($this, 'mceFullScreenButtonsStyle'));
		
		switch ($GLOBALS["pagenow"])
		{
			case "post.php":
			case "post-new.php":
				// init process for MCE button control
				add_action('init', array($this, 'add_mce_buttons'));
				add_action('admin_enqueue_scripts', array($this, 'beforeBuildRecipePage'), 1);
				add_action('admin_enqueue_scripts', array($this, 'buildRecipePageCallback'), 2);
				break;
		}
	}
	
	// Callback function registered for Media Uploader thickbox
	function my_admin_scripts()
	{
		wp_enqueue_script('media-upload');
		wp_enqueue_script('thickbox', null, array('jquery'));
		wp_register_script('my-upload', $this->kitchenbug->assets['js'] . '/kbug-wp.js', array('jquery', 'media-upload', 'thickbox'));
		wp_enqueue_script('my-upload');
	}

	// Callback function registered for  Media Uploader thickbox
	function my_admin_styles()
	{
		wp_enqueue_style('thickbox');
	}
	
	function beforeBuildRecipePage()
	{
		global $post;
		
		// Needed for the image editor lightbox
		wp_register_script('jQuery.tools', $this->siteURL . $this->kitchenbug->assets['js'] . '/jquery.tools.min.js', array('jquery'));
		wp_enqueue_script('jQuery.tools');
		
		// The editor form
		wp_register_script('kbug-recipe', $this->siteURL . $this->kitchenbug->assets['js'] . '/kbug-recipe.js', array('jquery'));
		// Pass PHP params to javascript
		$page = new KB_Pages_Recipe();
		$texts = $page->getPageData();             
		$settings = array('version' => $this->kitchenbug->config['plugin']['version'],
			'pluginURL' => $this->siteURL . $this->kitchenbug->pluginURL,
			'siteURL' => $this->siteURL,
			'ajaxURL' => admin_url('admin-ajax.php'),
			'configURL' => $this->kitchenbug->config['plugin']['url'],
			'editorPrefix' => $this->kitchenbug->editor->prefix->replaced,
			'permalink' => get_permalink($post->ID),
			'texts' => json_encode($texts)
			);
		wp_localize_script('kbug-recipe', 'kb_settings', $settings);
		wp_enqueue_script('kbug-recipe');
		
		// Editor form css
		wp_enqueue_style('kbug-recipe-editor', $this->siteURL . $this->kitchenbug->assets['css'] . '/kbug-recipe-editor.css');
	}
	
	// Callback function for enqueing all admin scripts
	function admin_init()
	{
		add_filter('plugin_action_links_' . 'kitchenbug/kitchenbug.php', array($this, '_pluginActionLinks'));
	}
	
	function add_mce_buttons()
	{
		add_filter('mce_external_plugins', array($this, 'mcePlugins'));
		add_filter('mce_buttons', array($this, 'mceButtons'));
		add_filter('wp_fullscreen_buttons', array($this, 'mceFullScreenButtons') );
	}

	/**
	 * Set default admin ajax
	 * @return void
	 */
	private function _setDefaultAjax()
	{
		$this->_addAjaxRequest('saveTags', array($this, 'saveTagsPage'));
		$this->_addAjaxRequest('updateUserId', array($this, 'updateUserId'));
		$this->_addAjaxRequest('kbaddView', array($this, 'addView'));
		$this->_addAjaxRequest('kbbackupFiles', array($this, 'backupFiles'));
		$this->_addAjaxRequest('kbUpdateSource', array($this, 'updateSource'));
		add_action('wp_ajax_nopriv_kbaddView', array($this, 'addView'));
	}

	public function backupFiles()
	{
		$recipePage = new KB_Pages_Recipe();
		
		$recipes = $recipePage->selectAll();
		if (count($recipes) == 0)
		{
			header("HTTP/1.1 500 Server Error");
			echo "No recipes in the Kitchenbug database.";
			exit();
		}
		
		$recipeXML = $this->_createRecipeXML($recipes);	
		// create file
		$upload_dir = wp_upload_dir();
		$filename = trailingslashit($upload_dir['path']).'recipes.xml';
		$fp = fopen($filename, "w");
		fputs ($fp, $recipeXML);
		fclose ($fp);
		
		header("HTTP/1.1 200 OK");
        echo $upload_dir['url'] .'/recipes.xml';
        exit();
	}
	
	private function _createRecipeXML($recipes)
	{
		// Create the xml document
		$xmlDoc = new DOMDocument();
		// Create the root element
		$root = $xmlDoc->appendChild($xmlDoc->createElement("Recipes"));
		// Iterate through the recipes
		foreach ($recipes as $recipe)
		{
			// Create a recipe element
			$recipeElement = $root->appendChild($xmlDoc->createElement("Recipe"));
			// Add recipe fields
			$recipeElement->appendChild($xmlDoc->createElement("RecipeID", $recipe['recipe_id']));
			$recipeElement->appendChild($xmlDoc->createElement("Name", $recipe['name']));
			$recipeElement->appendChild($xmlDoc->createElement("Description", $recipe['intro']));
			$recipeElement->appendChild($xmlDoc->createElement("PreparationTime", $recipe['prep_time']));
			$recipeElement->appendChild($xmlDoc->createElement("CookingTime", $recipe['cook_time']));
			$recipeElement->appendChild($xmlDoc->createElement("Servings", $recipe['servings']));
			$ingsElement = $recipeElement->appendChild($xmlDoc->createElement("Ingredients"));
			
			$ingredients = explode("\n",$recipe['ingredients'][0]);
			foreach ($ingredients as $ingredient)
			{
				$ingsElement->appendChild($xmlDoc->createElement("Ingredient", $ingredient));
			}
			$directionsElement = $recipeElement->appendChild($xmlDoc->createElement("Directions"));
			$directions = explode("\n",$recipe['directions'][0]);
			foreach ($directions as $direction)
			{
				$directionsElement->appendChild($xmlDoc->createElement("Direction", $direction));
			}
			$recipeElement->appendChild($xmlDoc->createElement("Image", $recipe['img_source']));
			$recipeElement->appendChild($xmlDoc->createElement("Tips", $recipe['tips']));
		}

		// Make the output pretty
		$xmlDoc->formatOutput = true;

		return $xmlDoc->saveXML();
	}

	public function updateUserId()
	{
		$page = new KB_Pages_Settings();
		$pageData = $page->save();
		die('0');
	}

	public function saveTagsPage()
	{
		$page = new KB_Pages_Recipe();
		$select = $page->save();
	
		echo sprintf($this->kitchenbug->editor->prefix->full, $select['recipe_id']);

		die();
	}
	
	public function updateSource()
	{
		
		// Save the source change so we don't go through this again
		$page = new KB_Pages_Recipe();
		$page->save();
		
		if (isset($_POST['post_id']))
		{
			$select = $page->select($_POST['post_id']);

			// Update source on server so we can find the analysis next time
			$result = KB_Http::getInstance()->request("http://". $this->kitchenbug->config['plugin']['url'] . "/analyzer/engine/source",
					true, array('recipe_id' => $select['recipe_id'], 'source' => urlencode($select['source']))
					);
		}
		
		die();
	}

	public function addView()
	{
		$result = KB_Http::getInstance()->request("http://". $this->kitchenbug->config['plugin']['url'] . "/admin/recipeviewed?recipe_id=" . 
										$_REQUEST['id'] . 
										"&view_type=blog&client_ip=" . 
										$_SERVER['REMOTE_ADDR']);

		if (isset($result['response']['message']))
		{
			echo $result['body'] . ' : (' . $_REQUEST['id'] . ')';
			exit();
		}
		else
		{
			echo $result;
			exit();
		}
	}

	public function mcePlugins($plugins)
	{
		$plugins = (array) $plugins;
		$plugins["kitchenbug"] = $this->siteURL . $this->kitchenbug->assets['js'] . '/kbug-mce.js?v=' . $this->version;
		return $plugins;
	}

	function mceButtons($buttons)
	{
		$buttons[] = 'separator';
		$buttons[] = 'kbugRecipe';
		return $buttons;
	}
	
	function mceFullScreenButtons($buttons)
	{
		$buttons[] = 'separator';
		$buttons['kitchenbug'] = array('title' => 'Kitchenbug', 
			'onclick' => "tinyMCE.execCommand('openkitchenbugeditor');", 
			'both' => false);
		return $buttons;
	}
	
	function mceFullScreenButtonsStyle()
	{
		wp_enqueue_style('fs-style', $this->siteURL . $this->kitchenbug->assets['css'] . '/kbug-fs-button.css');
	}

	public function settingsPageCallback()
	{
		$page = new KB_Pages_Settings;
		$pageData = null;

		foreach ($page->getPageData() as $key => $val)
		{
			if (!$key)
			{
				continue;
			}
			$this->view->$key = $val;
		}
		
		if (isset($_REQUEST['submit']))
		{
			$pageData = $page->save();
		}
		else
		{
			$pageData = $page->select();
		}
			

		if (!isset($pageData[KB_Pages_Settings::THEME]))
		{
			$pageData[KB_Pages_Settings::THEME] = KB_Pages_Settings::DEFAULT_THEME;
		}
		if (!empty($pageData))
		{
			foreach ($pageData as $setKey => $setVal)
			{
				if (!$setKey)
				{
					continue;
				}
				$this->view->$setKey = $setVal;
			}
		}
		
		$this->view->themes = $this->getThemes();

		echo $this->view->render('options-general');
	}

	public function getThemes()
	{
		$themes_path = ABSPATH . 'wp-content/plugins/kitchenbug/themes';

		$themes = array();

		if (is_dir($themes_path))
		{
			if ($dh = opendir($themes_path))
			{
				while (($file = readdir($dh)) !== false)
				{
					if ($file == '.' || $file == '..')
						continue;

					if (filetype($themes_path . '/' . $file) != 'dir')
						continue;

					if ($theme = $this->getThemeInfo($themes_path . '/' . $file, $file))
					{
						$themes[] = $theme;
					}
				}
				closedir($dh);
			}
		}

		return $themes;
	}

	private function getThemeInfo($theme_dir, $theme_slug)
	{
		if (!file_exists($theme_dir . '/style.css')) return null;

		$theme_info = array();

		if (is_dir($theme_dir))
		{
			if ($dh = opendir($theme_dir))
			{
				while (($file = readdir($dh)) !== false)
				{
					// Skip over system folders.
					if ($file == '.' || $file == '..') continue;

					// Get the contents of the style.css file, from which we will retrieve the
					// theme's info.
					$contents = file_get_contents($theme_dir . '/style.css');

					// Get the theme info from the style.css contents.
					if (!preg_match("|(?s)/\*(.*?)\*/|i", $contents, $matches)) return null;

					// Retrieve the different info fields from the theme info section.
					$theme_info['slug'] = $theme_slug;
					$theme_info['theme_name'] = $this->getThemeAttribute($matches[1], 'theme name');
					$theme_info['author'] = $this->getThemeAttribute($matches[1], 'author');
					$theme_info['author_uri'] = $this->getThemeAttribute($matches[1], 'author uri');
					$theme_info['description'] = $this->getThemeAttribute($matches[1], 'description');
					$theme_info['version'] = $this->getThemeAttribute($matches[1], 'version');
					$theme_info['path'] = $theme_dir;
				}
				closedir($dh);

				return $theme_info;
			}
		}
		
		return null;
	}

	private function getThemeAttribute($theme_header, $attribute)
	{
		preg_match("|(?s)$attribute: ([^\r\n]*)|i", $theme_header, $matches);
		return $matches[1];
	}
	
	public function addAdminMenu()
	{
		add_options_page($this->kitchenbug->config['plugin']['page_title'], $this->kitchenbug->config['plugin']['menu_title'], $this->kitchenbug->config['plugin']['capability'], $this->kitchenbug->config['plugin']['menu_title'], array($this, 'settingsPageCallback'));
	}

	public function _pluginActionLinks($links)
	{
		return array_merge(
			array('settings' => sprintf( '<a href="%s">%s</a>', self::GENERAL_OPTIONS_PAGE . '?page=Kitchenbug', __('Settings', 'Kitchenbug'))),
			$links
		);
		return $links;
	}

}