<?php

require_once('Db.php');
require_once('Pages/Settings.php');

class KB_Main
{

	public $paths = array();
	public $config;
	public $pluginURL;
	public $editor;
	public $themeNames = array();
	static private $_instance = false;

	const CONFIG_FILE = 'config.php';
	const AJAX_PAGE = 'admin-ajax.php';

	public static function getInstance($pluginPath = null)
	{
		if (!self::$_instance)
		{
			self::$_instance = new self($pluginPath);
		}
		return self::$_instance;
	}

	private function __construct($pluginPath)
	{
		$this->paths['include'] = self::getIncludePath();
		$this->paths['application'] = $this->paths['include'] . '/../application';
		$this->pluginURL = '/wp-content/plugins/' . $pluginPath;
		$this->themes = /* ABSPATH. */ $this->pluginURL . '/themes';
		$this->themesPath = ABSPATH . $this->pluginURL . '/themes';
		$assets = $this->pluginURL . '/application/assets';
		$this->assets = array(
			'js' => $assets . '/js',
			'css' => $assets . '/css',
			'img' => $assets . '/img',
		);

		$this->_getConfig();
		$this->_setEditorPrefix();

		add_action('admin_init', array($this, 'upgradePlugin'));
	}

	private function _setEditorPrefix()
	{
		$this->editor = new stdClass;
		$this->editor->prefix = array(
			'full' => $this->config['plugin']['editor-prefix'],
			'replaced' => str_replace('%d]', '', $this->config['plugin']['editor-prefix'])
		);
		$this->editor->prefix = (object) $this->editor->prefix;

		$this->placeholder = $this->config['plugin']['editor-placeholder'];
		$this->placeholder = str_replace('%s', get_site_url(), $this->placeholder);
	}

	public function InitializePlugin()
	{
		$settings = new KB_Pages_Settings();
		$settings->saveOption(array('KBPluginVersion' => $this->config['plugin']['version'],
				'nutritionOn' => 'on', 'excerptOn' => 'off', 'theme' => 'starter'));
	}

	public function upgradePlugin()
	{
		// Check if we need to upgrade
		$settings = new KB_Pages_Settings();
		$settingsData = $settings->select();
		if (isset($settingsData['KBPluginVersion']) && $settingsData['KBPluginVersion'] !== '0.5.6')
		{
			// Upgrade DB if 0.5.2, 0.5.3 or 0.5.4 version installed
			if ($settingsData['KBPluginVersion'] == '0.5.2' || $settingsData['KBPluginVersion'] == '0.5.3'
			|| $settingsData['KBPluginVersion'] == '0.5.4')
			{
				global $wpdb;
				$recipes_table = $wpdb->prefix . "kb_recipes";
				// Verify table exists
				if ($wpdb->get_var("SHOW TABLES LIKE '$recipes_table'") == $recipes_table)
				{
					$sql = "CREATE TABLE " . $recipes_table . " (
						post_id BIGINT(20) UNSIGNED NOT NULL UNIQUE KEY,
						content MEDIUMTEXT,
						engine_version int(7),
						kb_plugin_version varchar(20)
						)DEFAULT CHARSET=utf8;";

					require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
					dbDelta($sql);
				}
			}
			
			// Update settings
			if (!isset($settingsData['nutritionOn']))
			{
				$settingsData['nutritionOn'] = 'on';
			}
			
			if (!isset($settingsData['excerptOn']))
			{
				$settingsData['excerptOn'] = 'off';
			}
			
			$settingsData['KBPluginVersion'] = $this->config['plugin']['version'];
			$settings->saveOption($settingsData);
		}
	}

	public function getEditorPrefix()
	{
		return $this->editor->prefix->full;
	}

	public function getEditorPrefixReplaced()
	{
		return $this->editor->prefix->replaced;
	}

	public function getEditorPlaceHolder()
	{
		return $this->placeholder;
	}

	private function _getConfig()
	{
		$this->config = null;
		$this->config = @include_once($this->paths['application'] . '/config/' . self::CONFIG_FILE);

		//Define APPLICATION_ENV
		@include_once($_SERVER['DOCUMENT_ROOT'] . '/../../non_repo_configs.php');

		if (isset($_wp_object))
		{
			if (get_option('siteurl') != $_wp_object['db'])
			{
				update_option('siteurl', $_wp_object['db']);
				update_option('home', $_wp_object['db']);
			}
			unset($_wp_object['db']);
			// Parse with sections
			$this->config = self::array_merge_recursive_helper($_wp_object, $this->config);

			unset($GLOBALS['_wp_object']);
		}
	}

	public static function array_merge_recursive_helper(array $arr1, array $arr2)
	{
		foreach ($arr1 as $key => $value)
		{
			if (!isset($arr2[$key]))
			{
				$arr2[$key] = $value;
				continue;
			}

			if (!is_array($value))
			{
				$arr2[$key] = $arr1[$key];
				continue;
			}

			$arr2[$key] = self::array_merge_recursive_helper($arr1[$key], $arr2[$key]);
		}
		return $arr2;
	}

	public static function isAjaxCall()
	{
		if (self::AJAX_PAGE == $GLOBALS["pagenow"])
		{
			if (isset($_REQUEST["action"]))
			{
				return true;
			}
		}
		return false;
	}

	public static function getIncludePath()
	{
		return dirname(__FILE__);
	}

	public static function isURL($text)
	{
		return preg_match('@(https?://([-\w\.]+)+(:\d+)?(/([\w/_\.]*(\?\S+)?)?)?)@', $text);
	}

	public function __set($name, $value)
	{
		$this->$name = $value;
	}

	public function __get($name)
	{
		if (isset($this->$name))
		{
			return $this->$name;
		}
	}
}

/**
 * Fix for PHP V < 5.3
 * get_called_class
 */
if (!function_exists('get_called_class'))
{

	function get_called_class()
	{
		$bt = debug_backtrace();
		$l = 0;
		do
		{
			$l++;
			$lines = file($bt[$l]['file']);
			$callerLine = $lines[$bt[$l]['line'] - 1];
			preg_match('/([a-zA-Z0-9\_]+)::' . $bt[$l]['function'] . '/', $callerLine, $matches);

			if ($matches[1] == 'self')
			{
				$line = $bt[$l]['line'] - 1;
				while ($line > 0 && strpos($lines[$line], 'class') === false)
				{
					$line--;
				}
				preg_match('/class[\s]+(.+?)[\s]+/si', $lines[$line], $matches);
			}
		}
		while ($matches[1] == 'parent' && $matches[1]);
		return $matches[1];
	}

}