<?php

// Utility for converting object to array
function object_to_array($data)
{
	if ((!is_array($data)) and (!is_object($data)))
		return 'xxx'; //$data;

	$result = array();

	$data = (array) $data;
	foreach ($data as $key => $value)
	{
		if (is_object($value))
			$value = (array) $value;
		if (is_array($value))
			$result[$key] = object_to_array($value);
		else
			$result[$key] = $value;
	}

	return $result;
}

class KB_Pages_Recipe
{
	public $postID;
	protected $_tableName = 'kb_recipes';
	const POST_NAME = 'recipe';

	final public function __construct()
	{
		$this->postID = (int) @$_POST['post_id'];
		if ($this->postID == null)
		{
			$this->postID = (int) @$_GET['post'];
		}
		
		$this->lang = new KB_Lang;
	}

	/**
	 * Translate Texts with KB_Lang instance (just more comfortable to use)
	 * @param string $name
	 * @return string
	 */
	final protected function _translate($name)
	{
		return $this->lang->translate($name);
	}

	public function getPost($name)
	{
		if (!$name)
		{
			return $_POST;
		}

		if (empty($_POST[$name]))
		{
			return false;
		}
		$strPost = json_decode(stripslashes($_POST[$name]), TRUE);
		
		foreach ($strPost[$name] as $key => &$val)
		{
			if (is_array($val))
			{
				unset($val);
				continue;
			}

			$val = htmlspecialchars(stripslashes($val));
		}

		return $strPost;
	}

	/**
	 * Save post data & query current 
	 * @return Array
	 */
	public function save()
	{
		$post = $this->getPost(self::POST_NAME);
		$select = $this->select($this->postID);

		if (empty($post) || !is_array($post))
		{
			return isset($select) ? $select : false;
		}

		if (null === $select)
		{
			$this->insert(array($this->postID => $post['recipe']));
		}
		else
		{
			$data[$this->postID] = array_merge($select, $post['recipe']);
			$this->update($data);
		}

		return $post['recipe'];
	}

	public function selectAll()
	{
		global $wpdb;
		$results = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . $this->_tableName);
		$recipes = array();
		
		
		foreach ($results as $recipe)
		{
			$recipes[] = $this->convertToDatafromDB($recipe);
		}
		
		return $recipes;
	}
	
	public function select($postId)
	{
		global $wpdb;
		$recipe = $wpdb->get_row("SELECT * FROM " . $wpdb->prefix . $this->_tableName . ' WHERE post_id=' . $postId);

		return $this->convertToDatafromDB($recipe);
	}

	private function convertToDatafromDB($dbData)
	{
		if ($dbData == null)
		{
			return null;
		}
		//$data=array($this->postID=>json_decode($dbData->content));
		$data = json_decode($dbData->content);
		$data = object_to_array($data);

		return $data;
	}

	private function convertToDBStructurefromData($data)
	{
		if ($data == null)
		{
			return null;
		}
		$DBData = array('post_id' => $this->postID,
			'content' => json_encode($data[$this->postID])
		);
		return $DBData;
	}

	public function update($data)
	{
		global $wpdb;
		$wpdb->update(
				$wpdb->prefix . $this->_tableName, $this->convertToDBStructurefromData($data), array('post_id' => $this->postID), array(
			'%d', // value1
			'%s' // value2
				), array('%d')
		);
	}

	public function insert($data)
	{

		global $wpdb;
		$data = $this->convertToDBStructurefromData($data);
		$wpdb->insert(
				$wpdb->prefix . $this->_tableName, $data, array(
			'%d',
			'%s'
				)
		);
	}

	public function getPageData()
	{
		$page = new stdClass();

		$page->recipeName = $this->_translate('Recipe Name');
		$page->shortDescription = $this->_translate('Short Description');
		$page->prepTime = $this->_translate('Prep Time');
		$page->cookTime = $this->_translate('Cook Time');
		$page->Yield = $this->_translate('Servings');
		$page->uploadImageTitle = $this->_translate('WordPress Gallery Image');
		$page->uploadImage = $this->_translate('Open Gallery');
		$page->uploadImageInstructions = $this->_translate("Choose an image from the gallery and click on `Insert into Post`");
		$page->ingredients = $this->_translate('Ingredients');
		$page->directions = $this->_translate('Directions');
		$page->cookingTips = $this->_translate('Cooking Tips');
		$page->tags = $this->_translate('Tags');
		$page->autoTagsInstructionsBody = $this->_translate('Click on auto tag to add to list');
		$page->autoTagsInstructionsHead = $this->_translate('Auto Tags');
		$page->noTagsFound = $this->_translate('No tags found.');
		$page->continue = $this->_translate('Continue');
		$page->addRecipeToPost = $this->_translate('Add recipe to post');
		$page->usefullTips = $this->_translate('Useful tips');
		$page->forceContinue = $this->_translate('Skip');
		$page->tryAgain = $this->_translate('Try Again');
		$page->category = $this->_translate('Category');
		$page->cuisine = $this->_translate('Cuisine');
		$page->printAlternative = $this->_translate('You print - we plant!');
		$page->digInAlternative = $this->_translate('Detailed nutritional info');
		$page->showNutritionalValues = $this->_translate('Show nutritional values for the recipe');

		$page->cuisines = array(
			1 => 'American',
			2 => 'Asian',
			26 => 'Australian',
			3 => 'BBQ',
			4 => 'Cajun',
			5 => 'Canadian',
			6 => 'Chinese',
			7 => 'English and Irish',
			8 => 'French',
			9 => 'German',
			10 => 'Hungarian',
			11 => 'Indian',
			12 => 'Italian',
			13 => 'Jamaican',
			14 => 'Japanese',
			15 => 'Mediterranean or Greek',
			16 => 'Mexican',
			17 => 'Moroccan',
			18 => 'Portuguese',
			19 => 'Russian',
			20 => 'Scandinavian',
			21 => 'Soul food',
			22 => 'Spanish',
			23 => 'Swiss',
			24 => 'Thai',
			25 => 'Vietnamese',
		);

		$page->categories = array(
			1 => 'Appetizers',
			2 => 'Beverages',
			3 => 'Breakfast and Brunch',
			10 => 'Cakes and Cookies',
			4 => 'Condiments and Sauces',
			5 => 'Desserts',
			6 => 'Main Dishes',
			7 => 'Salads',
			8 => 'Side Dishes',
			9 => 'Soups',
		);

		$page->pTips = array(
			1 => $this->_translate('pTip1'),
			2 => $this->_translate('pTip2'),
			3 => $this->_translate('pTip3'),
			4 => $this->_translate('pTip4'),
			5 => $this->_translate('pTip5'),
		);

		return array(
			'analyzing' => $this->_translate('Analyzing...'),
			'savingRecipe' => $this->_translate('Saving Recipe...'),
			'updatingRecipe' => $this->_translate('Updating Recipe...'),
			'page' => $page,
		);
	}

	public function delete()
	{
		if (!isset($_GET['post']))
		{
			return false;
		}

		$postID = (int) $_GET['post'];

		global $wpdb;

		$recipe = $wpdb->query("DELETE FROM " . $wpdb->prefix . $this->_tableName . " where post_id = '" . $postID . "'");

		return true;
	}

}